function [loglik] = filter_cdkf(sysm,dim,data)
% Evaluating log-likelihood function by central difference Kalman filter

global ix1 ix2
nx = dim.x;
ix1 = 1:dim.x1;
ix2 = dim.x1+1:dim.x;

h = sqrt(3);

% initials
T = size(data,1);
loglik = zeros(T,1);
 
% initial value of s at t=0
Sigma = solve_cov(sysm);
Sw = sysm.R;
Sv = sysm.H;

Exf = zeros(nx,1);
Sxf = zeros(nx,nx);
Sxf(ix2,ix2) = chol(Sigma)';

% log likelihood by Kalman filter
for t = 1:T
    
    S1xx = fun_h2(sysm,Exf(:,ones(1,nx)),h*Sxf)/(2*h);
    S2xx = fun_h3(sysm,h*Sxf)*(sqrt(h^2-1))/(2*h^2);
    
    % forecasting
    Exp = (h^2-nx)/(h^2)*fun_h(sysm,Exf) + sum(fun_h1(sysm,Exf,h*Sxf),2)/(2*h^2);
    Sxp =  triag([S1xx,Sw,S2xx]);

    Eyp = (h^2-nx)/(h^2)*(fun_g(sysm,Exp)) + sum(fun_g1(sysm,Exp,h*Sxp),2)/(2*h^2);
    
    % filtering
    S1yx = fun_g2(sysm,Exp(:,ones(1,nx)),h*Sxp)/(2*h);
    S2yx = fun_g3(sysm,h*Sxp)*(sqrt(h^2-1))/(2*h^2);
    
    Syp =  triag([S1yx,Sv,S2yx]);

    Pyp = Syp*Syp';
    K = Sxp*S1yx'/Pyp;
    et = data(t,:)' - Eyp;
    Exf = Exp + K*et;
    Sxf = triag([Sxp-K*S1yx,K*Sv,K*S2yx]);
                   
	% log likelihood
    loglik(t) = -0.5*dim.y*log(2*pi) - 0.5*log(det(Pyp)) - 0.5*et'/Pyp*et;

end
loglik = sum(loglik);
end

function y = fun_g(sysm,x)
N = size(x,2);
z = sysm.gc(:,ones(N,1)) + sysm.gl*x + qf(sysm.gq,x); 
y = sysm.c(:,ones(N,1)) + sysm.zz*z + sysm.zx*x;
end

function y = fun_g1(sysm,x,s)
N = size(s,2);
z = sysm.gc + sysm.gl*x + qf(sysm.gq,x);
z = z(:,ones(N,1)) + qf(sysm.gq,s);
y = sysm.c + sysm.zx*x;
y = 2*(y(:,ones(N,1)) + sysm.zz*z);
end

function y = fun_g2(sysm,x,s)
z = sysm.gl*s + qf(sysm.gq,x,s) + qf(sysm.gq,s,x);
y = 2*(sysm.zz*z + sysm.zx*s);
end

function y = fun_g3(sysm,s)
y = 2*sysm.zz*qf(sysm.gq,s);
end

function y = fun_h(sysm,x)
global ix1
N = size(x,2);
y = sysm.hc(:,ones(N,1)) + sysm.hl*x;
y(ix1,:) = y(ix1,:) + qf(sysm.hq,x);
end

function y = fun_h1(sysm,x,s)
global ix1
N = size(s,2);
y = sysm.hc + sysm.hl*x;
y = 2*y(:,ones(N,1));
d = qf(sysm.hq,x);
y(ix1,:) = y(ix1,:) + 2*(d(:,ones(N,1))+qf(sysm.hq,s));  
end

function y = fun_h2(sysm,x,s)
global ix1
y = 2*sysm.hl*s;
y(ix1,:) = y(ix1,:) + 2*(qf(sysm.hq,x,s)+qf(sysm.hq,s,x));  
end

function y = fun_h3(sysm,s)
global ix1
[nx,N] = size(s);
y = zeros(nx,N);
y(ix1,:) = 2*qf(sysm.hq,s);
end

function y = qf(Q,x,s)
if nargin == 2
    s = x;
end
[nx,N] = size(x);
ny = size(Q,1)/nx;
a = Q*x;
b = a.*repmat(s,ny,1);
c = sum(reshape(b,nx,ny*N),1);
y = reshape(c,ny,N);
end

function[G0] = solve_cov(sysm)
global dim ix2
nx2 = dim.x2;
Sigmau = sysm.R(ix2,:)*sysm.R(ix2,:)';
G0v = (eye(nx2^2)-kron(sysm.hl(ix2,ix2),sysm.hl(ix2,ix2)))\Sigmau(1:nx2^2)';
G0  = zeros(nx2,nx2);
G0(1:nx2^2) = G0v;
end
