function [loglik,weight] = filter_cdpf(sysm,dim,data,N)
% Evaluating log-likelihood function by auxiliary particle filtering
% Linearizing measurement equation around E(x_t|I_{t-1},y_t) (this expectation
% is approximated by CDKF updating)

global ix1 ix2 
ix1 = 1:dim.x1;
ix2 = dim.x1+1:dim.x;

R2 = sysm.R*sysm.R';    % covariance of structural shocks
H2 = sysm.H*sysm.H';    % covariance of measurement errors

% initials
T = size(data,1);
loglik = 1:T;
h = sqrt(3);

% initial value of s at t=0
Sigma = solve_cov(sysm,dim);
Sxf = zeros(dim.x,dim.x);
Sxf(ix2,ix2) = triag(chol(Sigma)');

xt = zeros(dim.x,N);
xt(ix2,:) = Sxf(ix2,ix2)*randn(dim.u,N);
Exf = zeros(dim.x,1);

df_t = 20;

%ess = 1:T;
weight = nan(N,T);
for t = 1:T
    yt = data(t,:)';
    [Exf,Sxf] = cdkf_upda(sysm,dim,Exf,Sxf,h,yt);  %
    Pxf = Sxf*Sxf';
    Exp = fun_h(sysm,xt);  
    xt(ix1,:) = Exp(ix1,:);
    e3 = Sxf(ix2,ix2)*repmat(1./sqrt(gamrnd(df_t/2,2/df_t,1,N)),dim.u,1).*randn(dim.u,N);
    xt(ix2,:) = Exf(ix2,ones(1,N)) + e3;
             
    % evaluates importance weights
    e1 = xt(ix2,:) - Exp(ix2,:);
    e2 = yt(:,ones(N,1)) - fun_g(sysm,xt);
    lnwt = lmnpdf2(e1,R2(ix2,ix2)) + lmnpdf2(e2,H2) - lmtpdf2(e3,Pxf(ix2,ix2),df_t);
    dfac = max(lnwt);
    wt   = exp(lnwt-dfac);
    loglik(t) = log(sum(wt)) - log(N) + dfac;
   
    weight(:,t) = wt;
    %cv = sum((wt-mean(wt)).^2)/(N-1)/mean(wt)^2;
    %ess(t) = N/(1+cv);
    Exf = xt*wt/sum(wt);
    
    xt = resmpl_mul(xt,wt/sum(wt));   
    
end
loglik = sum(loglik);
end


%--------------------------------------------------------------------------
function [Ex,Sxf] = cdkf_upda(sysm,dim,Exf,Sxf,h,y)
nx = dim.x;

Sw = sysm.R;
Sv = sysm.H;

S1xx = fun_h2(sysm,Exf(:,ones(1,nx)),h*Sxf)/(2*h);
S2xx = fun_h3(sysm,h*Sxf)*(sqrt(h^2-1))/(2*h^2);

Exp = (h^2-nx)/(h^2)*fun_h(sysm,Exf) + sum(fun_h1(sysm,Exf,h*Sxf),2)/(2*h^2);
Sxp = triag([S1xx,Sw,S2xx]);
Eyp = (h^2-nx)/(h^2)*(fun_g(sysm,Exp)) + sum(fun_g1(sysm,Exp,h*Sxp),2)/(2*h^2);
    
% filtering
S1yx = fun_g2(sysm,Exp(:,ones(1,nx)),h*Sxp)/(2*h);
S2yx = fun_g3(sysm,h*Sxp)*(sqrt(h^2-1))/(2*h^2);

Syp = triag([S1yx,Sv,S2yx]);
Pyp = Syp*Syp';
K = Sxp*S1yx'/Pyp;
    
et = y - Eyp;
Ex = Exp + K*et;
Sxf = triag([Sxp-K*S1yx,K*Sv,K*S2yx]);
end

function y = fun_g(sysm,x)
N = size(x,2);
z = sysm.gc(:,ones(N,1)) + sysm.gl*x + qf(sysm.gq,x); 
y = sysm.c(:,ones(N,1)) + sysm.zz*z + sysm.zx*x;
end

function y = fun_g1(sysm,x,s)
N = size(s,2);
z = sysm.gc + sysm.gl*x + qf(sysm.gq,x);
z = z(:,ones(N,1)) + qf(sysm.gq,s);
y = sysm.c + sysm.zx*x;
y = 2*(y(:,ones(N,1)) + sysm.zz*z);
end

function y = fun_g2(sysm,x,s)
z = sysm.gl*s + qf(sysm.gq,x,s) + qf(sysm.gq,s,x);
y = 2*(sysm.zz*z + sysm.zx*s);
end

function y = fun_g3(sysm,s)
y = 2*sysm.zz*qf(sysm.gq,s);
end

function y = fun_h(sysm,x)
global ix1
N = size(x,2);
y = sysm.hc(:,ones(N,1)) + sysm.hl*x;
y(ix1,:) = y(ix1,:) + qf(sysm.hq,x);
end

function y = fun_h1(sysm,x,s)
global ix1
N = size(s,2);
y = sysm.hc + sysm.hl*x;
y = 2*y(:,ones(N,1));
d = qf(sysm.hq,x);
y(ix1,:) = y(ix1,:) + 2*(d(:,ones(N,1))+qf(sysm.hq,s));  
end

function y = fun_h2(sysm,x,s)
global ix1
y = 2*sysm.hl*s;
y(ix1,:) = y(ix1,:) + 2*(qf(sysm.hq,x,s)+qf(sysm.hq,s,x));  
end

function y = fun_h3(sysm,s)
global ix1
[nx,N] = size(s);
y = zeros(nx,N);
y(ix1,:) = 2*qf(sysm.hq,s);
end

function y = qf(Q,x,s)
if nargin == 2
    s = x;
end
[nx,N] = size(x);
ny = size(Q,1)/nx;
a = Q*x;
b = a.*repmat(s,ny,1);
c = sum(reshape(b,nx,ny*N),1);
y = reshape(c,ny,N);
end

function[G0] = solve_cov(sysm,dim)
global ix2
nx2 = dim.x2;
Sigmau = sysm.R(ix2,:)*sysm.R(ix2,:)';
G0v = (eye(nx2^2)-kron(sysm.hl(ix2,ix2),sysm.hl(ix2,ix2)))\Sigmau(1:nx2^2)';
G0  = zeros(nx2,nx2);
G0(1:nx2^2) = G0v;
end

function [y,ind]= resmpl_mul(x,w)
% Y=RESMPL(X,W)
% resamples from X with weights W, with replacement
% based on multinomial sampling
N = size(x,2);
ind = randsample(1:N,N,true,w);
y = x(:,ind);
end