function [loglik,weight] = filter_cpf(sysm,dim,data,N)
% Evaluating log-likelihood function by conditional particle filtering
% Optimal conditioanl sampling distribution is approximated by linearizing
% measurement equation at E(x_t|I_{1:t-1})

global ix1 ix2 
nx = dim.x;
ix1 = 1:dim.x1;
ix2 = dim.x1+1:dim.x;

R2 = sysm.R*sysm.R';    % covariance of structural shocks
H2 = sysm.H*sysm.H';    % covariance of measurement errors

% initials
T = size(data,1);
loglik = 1:T;

% initial value of s at t=0
Sigma = solve_cov(sysm,dim);
xt = zeros(nx,N);
xt(ix2,:) = chol(Sigma)'*randn(dim.u,N);
Sxp = zeros(5,5);
Sxp(3:5,3:5) = sqrt(R2(3:5,3:5));

%ess = 1:T;
%Exf = nan(nx,T);
weight = nan(N,T);
for t = 1:T
    
    yt = data(t,:)';
    
    Exp = fun_h(sysm,xt);                    % E(x_t|x_{t-1})
    x0 = mean(Exp,2);
    Dg = gre_g(sysm,dim,x0);
    aux1 = fun_g(sysm,x0) - Dg*x0;   % auxiliary variable 1
    Eyp = aux1(:,ones(1,N)) + Dg*Exp; 
    
    A = triag([Sxp,zeros(nx,dim.e);Dg*Sxp,sysm.H]);
    Sxf = A(1:nx,1:nx);
    K = A(1:nx,nx+1:end);
    Sy = A(nx+1:end,nx+1:end);
    K = K*Sy'/(Sy*Sy');
    
    aux2 = K*yt;    % auxiliary variable 2    
    Ex = Exp + aux2(:,ones(1,N)) - K*Eyp;
    Px = Sxf*Sxf';
    
    % sampling
    e3 = Sxf(ix2,ix2)*randn(dim.u,N);
    xt(ix1,:) = Exp(ix1,:);          
    xt(ix2,:) = Ex(ix2,:) + e3;
    e1 = xt(ix2,:) - Exp(ix2,:);
    e2 = yt(:,ones(N,1)) - fun_g(sysm,xt);  % p(y_t|x_t)   
        
    lnwt = lmnpdf2(e1,R2(ix2,ix2)) + lmnpdf2(e2,H2) - lmnpdf2(e3,Px(ix2,ix2));
    dfac = max(lnwt);
    wt   = exp(lnwt-dfac);
    loglik(t) = log(sum(wt)) - log(N) + dfac;
    weight(:,t) = wt;
   
    %cv = sum((wt-mean(wt)).^2)/(N-1)/mean(wt)^2;
    %ess(t) = N/(1+cv);
    %Exf(:,t) = xt*wt/sum(wt);
    
    xt = resmpl_mul(xt,wt/sum(wt));   
    
end
loglik = sum(loglik);
end

function y = fun_g(sysm,x)
N = size(x,2);
z = sysm.gc(:,ones(N,1)) + sysm.gl*x + qf(sysm.gq,x); 
y = sysm.c(:,ones(N,1)) + sysm.zz*z + sysm.zx*x;
end

function y = fun_h(sysm,x)
global ix1
N = size(x,2);
y = sysm.hc(:,ones(N,1)) + sysm.hl*x;
y(ix1,:) = y(ix1,:) + qf(sysm.hq,x);
end

function y = qf(Q,x,s)
if nargin == 2
    s = x;
end
[nx,N] = size(x);
ny = size(Q,1)/nx;
a = Q*x;
b = a.*repmat(s,ny,1);
c = sum(reshape(b,nx,ny*N),1);
y = reshape(c,ny,N);
end

function[G0] = solve_cov(sysm,dim)
global ix2
nx2 = dim.x2;
Sigmau = sysm.R(ix2,:)*sysm.R(ix2,:)';
G0v = (eye(nx2^2)-kron(sysm.hl(ix2,ix2),sysm.hl(ix2,ix2)))\Sigmau(1:nx2^2)';
G0  = zeros(nx2,nx2);
G0(1:nx2^2) = G0v;
end

function y = gre_g(sysm,dim,x)
nx = dim.x;
nz = dim.z;
N = size(x,2);
y = sysm.zz*sysm.gl + sysm.zx;
a = sysm.gq*x;
b = reshape(a,nx,nz*N);
y = y + 2*sysm.zz*b'; 
end

function [y,ind]= resmpl_mul(x,w)
% Y=RESMPL(X,W)
% resamples from X with weights W, with replacement
% based on multinomial sampling
N = size(x,2);
ind = randsample(1:N,N,true,w);
y = x(:,ind);
end